from sencore import PhraseParser
from x2cdict import PhraseDict


class PhraseAnalyzer:
  """Analyze phrase of sentences 
  """
  def __init__(self, lang):
    """Initialize ``parser`` and ``dictAPI`` by language

    Args:
      lang (str): language abbreviation, e.g, ``en``, ``es``
    """

    self._parser = PhraseParser(lang)
    self._dictAPI = PhraseDict(lang, "cn")

  def _phrases(self, sens):
    ps = []
    for s in sens:
      phrases = self._parser.digest(s)
      phrases["sentence"] = s
      ps.append(phrases)
    return ps

  def _explain_sentence(self, sentence):
    result = {"text": sentence, "meaning": sentence}
    ex = self._dictAPI.search(sentence)
    if ex:
      result["meaning"] = ex["translated"]
    return result

  def _explain_default_phrases(self, phrases):
    result = []
    for p in phrases:
      ex = self._dictAPI.search(p)
      if ex:
        _r = {"text": p, "meaning": ex["translated"]}
        result.append(_r)
    return result
      
  def overview_phrases(self, sens):
    """Get phrases with explanation from sentences.

    Args:
      sens (list): sentences

    Returns:
      list of dict: one element contains phrases for each sentence. one element has keys: ``sentence``, ``noun_phrases``, ``prep_phrases``, ``verb_phrases``, ``verbs`` 
    """

    result = []
    ps = self._phrases(sens)
    for p in ps:
      sentence_obj = self._explain_sentence(p["sentence"])
      nps_obj = self._explain_default_phrases(p["noun_phrases"])
      vs_obj = p["verbs"]
    
      r = {
        "sentence": sentence_obj,
        "noun_phrases": nps_obj,
        "verbs": vs_obj
      }
      result.append(r)
    return result




from sencore import StructureParser, KGParser
from sencore.lib import explain
from x2cdict import PhraseDict

def add_bracket(ex):
  return " (" + ex + ")" if ex!=None else "" 

class StructureKGAnalyzer:
  """Analyze sentence structure and kg of sentences 
  """
  def __init__(self, lang):
    """Initialize ``kg_parser``, ``structure_parser`` and ``dictAPI`` by language

    Args:
      lang (str): language abbreviation, e.g, ``en``, ``es``
    """

    self._structure_parser = StructureParser(lang)
    self._kg_parser = KGParser(lang, labels=["VERB"])
    self._dictAPI = PhraseDict(lang, "cn")

  def _parse(self, sens):
    content = []
    for s in sens:
      structure = self._structure_parser.digest(s)
      kg = self._kg_parser.digest(s) 

      _data = {
        "sentence": s,
        "kg": kg,
        "structure": structure,
      }
      content.append(_data) 
    return content

  def _explain_sentence(self, sentence):
    result = {"text": sentence, "meaning": sentence}
    ex = self._dictAPI.search(sentence)
    if ex:
      result["meaning"] = ex["translated"]
    return result

  def _explain_kg(self, kg):
    kg_translator = self._kg_parser.get_translator("cn") 
    d = explain(kg, kg_translator)
    return d

  def _explain_structure(self, structure):
    for p in structure:
      if p["explanation"]:
        text = p["text"]
        p["meaning"] = text
        ex = self._dictAPI.search(text)
        if ex:
          p["meaning"] = ex["translated"]
    return structure 
 
  def _represent_structure(self, structure):
    rep = " ".join([t["text"] + add_bracket(t["explanation"]) for t in structure])
    return rep

  def overview_structure_kg(self, sens):
    """Get structure and kg with translation from sentences.

    Args:
      sens (list): sentences

    Returns:
      list of dict: one element contains kg and structure for each sentence. one element has keys: ``sentence``, ``kg``, ``structure`` 
    """

    result = []
    content = self._parse(sens)
    for p in content:
      sentence_obj = self._explain_sentence(p["sentence"])
      structure_obj = self._explain_structure(p["structure"])
      structure_rep = self._represent_structure(p["structure"])
      kg_obj = self._explain_kg(p["kg"])
    
      r = {
        "sentence": sentence_obj,
        "structure": structure_obj,
        "structure_rep": structure_rep,
        "kg": kg_obj,
      }
      result.append(r)
    return result




import spacy

class PlaintextReader:
  """Read plain text file

  Attributes:
    text (str): content of file
  """

  def __init__(self, source, stype, lang):
    """Initialize nlp package, file content.

    Args:
      source (str): raw text content or file uri according to ``stype``
      stype (str): source type, ``RAW`` or ``FILE``
      lang (str): language abbreviation
    """
    _pkg = "{}_dep_news_trf".format(lang) if lang != "en" else "en_core_web_trf"
    self._nlp = spacy.load(_pkg, disable=["tagger", "ner", "lemmatizer", "textcat"])
    self.text = ""

    if stype == "RAW":
      self.text = source
    elif stype == "FILE":
      with open(source, "r") as f:
        self.text = f.read().replace("\n", " ")
    else:
      raise Exception("stype can only be: RAW or FILE !!!")

  @property
  def sentences(self):
    """list of str: sentences in file content 
    """
    doc = self._nlp(self.text)
    sentences = [s.text for s in list(doc.sents) if s.text.strip() != ""]
    return sentences
